@@ -6,10 +6,19 @@ const app = angular.module('dataiku.agenttools', ['dataiku.services', 'dataiku.f
 app.controller('AgentToolController', function ($scope, $state, $stateParams, $q, $timeout, WT1, DataikuAPI, PromptUtils, TopNav, DatasetUtils, FutureProgressModal,
     $rootScope, FutureWatcher, Debounce, CreateModalFromTemplate, TaggingService, Dialogs, localStorageService, SmartId, translate, ClipboardUtils, AgentToolService,
     SavedModelsService, PluginsService, AgentToolCodeTemplates, ActiveProjectKey, Expressions, CodeBasedEditorUtils, DKUtils, RetrievableKnowledgeUtils,
-    CodeMirrorSettingService) {
+    CodeMirrorSettingService, CodeStudiosService, FLOW_COMPUTABLE_DEPENDENCIES, SavedModelHelperService) {
+    // inline python tools
     $scope.uiState = {};
 
-    // inline python tools
+    $scope.availableDependencies = [];
+    $scope.currentDependencies = [];
+
+    $scope.warningMsg = ''
+
+    $scope.isDependenciesEnabled = function(){
+        return $scope.agentTool.type === 'InlinePython';
+    }
+
     $scope.editorOptions = CodeBasedEditorUtils.editorOptions('text/x-python', $scope, true);
     CodeBasedEditorUtils.registerBroadcastSelectionHandler($scope.editorOptions);
     CodeBasedEditorUtils.setRecipeScript($scope.script);
@@ -24,6 +33,13 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
         savedAgentTool = angular.copy(agentTool);
     };
 
+    $scope.canEditCodeAgentInCodeStudio = false;
+    $scope.editAgentToolInCodeStudio = function() {
+        $scope.save().then(function() {
+            CodeStudiosService.editFileInCodeStudio($scope, "agent_tools", $scope.agentTool.id);
+        }).catch(setErrorInScope.bind($scope));
+    }
+
     $scope.isDirty = function() {
         return !angular.equals(savedAgentTool, $scope.agentTool);
     };
@@ -41,10 +57,13 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
         const deferred = $q.defer();
         const saveAfterConflictCheck = function() {
             DataikuAPI.agentTools.save($scope.agentTool).success(function(data) {
-                WT1.event(
-                    'agent-tool-save', {
-                        agentToolType: data.type,
-                    });
+                const wt1AgentToolSave = {
+                    agentToolType: data.type,
+                };
+                if ($scope.pluginDesc && $scope.pluginDesc.id) {
+                    wt1AgentToolSave.pluginId = $scope.pluginDesc.id;
+                }
+                WT1.event('agent-tool-save', wt1AgentToolSave);
                 $scope.agentTool = data;
                 savedAgentTool = angular.copy($scope.agentTool);
                 $scope.$broadcast('agentToolSavedEvent');
@@ -99,6 +118,35 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
         });
     };
 
+    $scope.addDependency = function(dependencyToAdd) {
+        if (!dependencyToAdd) return;
+
+        // Check if the dependency is already added to prevent duplicates
+        $scope.agentTool.params.dependencies = $scope.agentTool.params.dependencies || [];
+        const isAlreadyAdded = $scope.agentTool.params.dependencies.some(dep => dep.ref === dependencyToAdd.smartId);
+        if (isAlreadyAdded) {
+            $scope.warningMsg = "Object already added as dependency.";
+            return;
+        }
+
+        // Create and add the new dependency object
+        const newDependency = {
+            type: dependencyToAdd.type,
+            ref: dependencyToAdd.smartId
+        };
+        //BE object has type, ref only
+        $scope.agentTool.params.dependencies.push(newDependency);
+        //UI object
+        $scope.currentDependencies.push(dependencyToAdd);
+        $scope.warningMsg = "";
+    };
+
+    $scope.removeDependency = function(index) {
+        if (index < 0) { return; }
+        $scope.agentTool.params.dependencies.splice(index, 1);
+        $scope.currentDependencies.splice(index, 1);
+    };
+
     $scope.showToolDescriptionModal = function(agentTool){
         DataikuAPI.agentTools.getDescriptor(agentTool.projectKey, agentTool.id).success(descriptor => {
             const modalScope = $scope.$new(true);
@@ -110,16 +158,64 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
         }).error(setErrorInScope.bind($scope));
     }
 
+    $scope.getColumnDescriptionNameWarning = function(columnName) {
+        if (!$scope.knowledgeBankSchemaColumnNames) return null;
+        if (!$scope.knowledgeBankSchemaColumnNames.includes(columnName)) {
+            return `${columnName} not in metadata columns.`;
+        }
+        return null;
+    }
+
+    $scope.isToolRunWithKernelPool = function() {
+        const toolsRunWithKernelPool = ["VectorStoreSearch", "InlinePython", "GenericStdioMCPClient"];
+        return ($scope.agentTool.type.indexOf("Custom") == 0) || toolsRunWithKernelPool.includes($scope.agentTool.type);
+    };
+
+    $scope.listLogs = function() {
+        DataikuAPI.agentTools.listLogs($stateParams.projectKey, $scope.agentTool.id).success(function(data) {
+            $scope.agentToolLogs = data;
+        }).error(setErrorInScope.bind($scope));
+    }
+
+    $scope.clearLogs = function() {
+        const title = 'Confirm logs deletion';
+        const message = 'Are you sure you want to clear the logs for this agent tool?';
+
+        Dialogs.confirm($scope, title, message).then(() => {
+            DataikuAPI.agentTools.clearLogs($stateParams.projectKey, $scope.agentTool.id)
+                .error(setErrorInScope.bind($scope))
+                .finally($scope.listLogs);
+        });
+    }
+
+    $scope.deleteLog = function(projectKey, id, logName) {
+        return DataikuAPI.agentTools.deleteLog(projectKey, id, logName)
+            .error(setErrorInScope.bind($scope))
+            .finally($scope.listLogs);
+    }
+
+    $scope.getLog = DataikuAPI.agentTools.getLog;
+
+    $scope.logDownloadURL = function(projectKey, id, logName) {
+        const params = new URLSearchParams({
+            projectKey,
+            id,
+            logName
+        });
+        return `/dip/api/agent-tools/stream-log?${params}`;
+    }
+
     //
-    // Start of local MCP tool specific
+    // Start of MCP tool specific
     //
     $scope.showMCPJsonConfigModal = function(){
         CreateModalFromTemplate('/templates/agent-tools/agent-tool-mcp-json-modal.html', $scope);
     }
 
     $scope.subtools = undefined;
+    $scope.subtoolsCanBeEnabledByDefault = undefined;
     $scope.loadSubtools = function() {
-        if (!$scope.agentTool || $scope.agentTool.type !== "GenericStdioMCPClient" || !$scope.agentTool.params) return;
+        if (!$scope.agentTool || !["GenericStdioMCPClient", "RemoteMCPClient"].includes($scope.agentTool.type) || !$scope.agentTool.params) return;
         $scope.subtools = undefined;
         $scope.subtoolsError = undefined;
         $scope.loadingSubtools = true;
@@ -182,7 +278,7 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
     };
 
     $scope.toggleSubtool = function(tool, state) {
-        if (!$scope.agentTool || $scope.agentTool.type !== "GenericStdioMCPClient" || !$scope.agentTool.params) return;
+        if (!$scope.agentTool || !["GenericStdioMCPClient", "RemoteMCPClient"].includes($scope.agentTool.type) || !$scope.agentTool.params) return;
 
         if (state !== undefined) {
             tool.enabled = state;
@@ -191,7 +287,11 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
             if (tool.name in $scope.agentTool.params.subtoolsStateOverride) {
                 $scope.agentTool.params.subtoolsStateOverride[tool.name] = !$scope.agentTool.params.subtoolsStateOverride[tool.name];
             } else {
-                $scope.agentTool.params.subtoolsStateOverride[tool.name] = !$scope.agentTool.params.subtoolsEnabledByDefault;
+                if ($scope.subtoolsCanBeEnabledByDefault) {
+                    $scope.agentTool.params.subtoolsStateOverride[tool.name] = !$scope.agentTool.params.subtoolsEnabledByDefault;
+                } else {
+                    $scope.agentTool.params.subtoolsStateOverride[tool.name] = true;
+                }
             }
         }
     };
@@ -351,7 +451,7 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
     }
 
     //
-    // End of local MCP tool specific
+    // End of MCP tool specific
     //
 
     function load() {
@@ -367,19 +467,29 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
                 agentToolType: $scope.agentTool.type
             });
             TopNav.setPageTitle($scope.agentTool.name + " - Agent Tool");
-
-            WT1.event("agent-tool-load", {
+            
+            const wt1AgentToolLoad = {
                 agentToolType: $scope.agentTool.type,
-            });
+            };
 
-            if ($scope.agentTool.type.indexOf("Custom") == 0) {
-                $scope.customAgentToolDesc = $rootScope.appConfig.customAgentTools.findLast(c => c.agentType === $scope.agentTool.type.substring(7));
+            $scope.customAgentToolDesc = AgentToolService.getCustomAgentToolDesc($scope.agentTool.type);
+            if ($scope.customAgentToolDesc) {
                 $scope.pluginDesc = PluginsService.getOwnerPluginDesc($scope.customAgentToolDesc);
+                if ($scope.pluginDesc && $scope.pluginDesc.id) {
+                    wt1AgentToolLoad.pluginId = $scope.pluginDesc.id;
+                }
             }
 
+            WT1.event('agent-tool-load', wt1AgentToolLoad);
+
             if ($scope.agentTool.type == 'VectorStoreSearch') {
                 // keep in sync with retrievable-knowledge.js
                 let kbDefaults = RetrievableKnowledgeUtils.getRAGLLMSettings();
+                const dku_embedding_column_option = {
+                    value:"DKU_TEXT_EMBEDDING_COLUMN",
+                    label: "Embedding column (Always included)",
+                    disabled: true
+                }
                 if ($scope.agentTool.params === undefined) $scope.agentTool.params = [];
                 Object.keys(kbDefaults).forEach((field) => {
                     if ($scope.agentTool.params[field] === undefined)
@@ -390,10 +500,7 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
                     "columns" : []
                 };
                 $scope.knowledgeBankSchemaColumnNames = [];
-
-                $scope.getSelectableKBColumns = function() {
-                    return angular.copy($scope.knowledgeBankSchemaColumnNames);
-                };
+                $scope.retrievalColumnsOptions = [dku_embedding_column_option]
 
                 $scope.$watch("agentTool.params.knowledgeBankRef", Debounce().withScope($scope).withDelay(50,200).wrap(function(){
                     if (!($scope.agentTool && $scope.agentTool.params && $scope.agentTool.params.knowledgeBankRef)) return;
@@ -402,6 +509,7 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
                         $scope.knowledgeBank = retrievableKnowledge;
                         $scope.knowledgeBankSchema.columns =  $scope.knowledgeBank.metadataColumnsSchema || [];
                         $scope.knowledgeBankSchemaColumnNames = $scope.knowledgeBankSchema.columns.map((column) => column.name);
+                        $scope.retrievalColumnsOptions = $scope.retrievalColumnsOptions.concat($scope.knowledgeBankSchemaColumnNames.map(column => {return {value: column, label: column, disabled: false}}))
                     }).error(setErrorInScope.bind($scope));
 
                     DataikuAPI.retrievableknowledge.getFullInfo(ActiveProjectKey.get(), SmartId.create($scope.agentTool.params.knowledgeBankRef, $stateParams.projectKey)).then(function({data}){
@@ -621,6 +729,10 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
                     $scope.save();
                 }
 
+                DataikuAPI.codeStudioTemplates.canEditInCodeStudio($stateParams.projectKey).success(function(data) {
+                    $scope.canEditCodeAgentInCodeStudio = data.canEdit;
+                })
+
                 TopNav.setLocation(TopNav.TOP_AGENT_TOOLS, 'agent tools', TopNav.TABS_NONE, "design");
             }
 
@@ -724,6 +836,32 @@ app.controller('AgentToolController', function ($scope, $state, $stateParams, $q
                     }
                 }).error(setErrorInScope.bind($scope));
             }
+
+            if ($scope.agentTool.type === 'GenericStdioMCPClient') {
+                $scope.subtoolsCanBeEnabledByDefault = true;
+            }
+
+            if ($scope.agentTool.type === 'RemoteMCPClient') {
+                $scope.subtoolsCanBeEnabledByDefault = false;
+                DataikuAPI.connections.getNames("RemoteMCP")
+                    .success(function(data) {
+                        $scope.availableRemoteMCPConnections = data;
+                    })
+                    .error(setErrorInScope.bind($scope));
+
+                $scope.$watch("agentTool.params.connectionName", function(nv, ov) {
+                    if (!nv) return;
+                    $scope.loadSubtools();
+                }, true);
+            }
+
+            DataikuAPI.taggableObjects.listAccessibleObjects($stateParams.projectKey).then(function(resp) {
+                const dependencies = resp.data;
+                $scope.currentDependencies = SavedModelHelperService.enrichDependencies($scope.agentTool?.params?.dependencies || [], dependencies);
+                $scope.availableDependencies = SavedModelHelperService.filterDependencies(dependencies, FLOW_COMPUTABLE_DEPENDENCIES);
+            }).catch(setErrorInScope.bind($scope))
+
+            $scope.listLogs();
         }).error(setErrorInScope.bind($scope));
     }
 
@@ -759,13 +897,22 @@ app.controller("AgentToolsListController", function($scope, $controller, $stateP
         orderReversed: false
     }, $scope.selection || {});
 
+    const showForeignFilter = $rootScope.featureFlagEnabled('allowForeignInProjectObjectLists');
+    if(showForeignFilter) {
+        $scope.enableForeignFilterWithLabels({
+            all: 'All agent tools',
+            onlyLocal: 'Only agent tools from the project',
+            onlyForeign: 'Only shared agent tools',
+        });
+    }
     $scope.list = function() {
-        DataikuAPI.agentTools.listHeads($stateParams.projectKey).success(function(data) {
-            $scope.listItems = data;
-            $scope.restoreOriginalSelection();
-        }).error(setErrorInScope.bind($scope));
-
-        DataikuAPI.agentTools.list($stateParams.projectKey).success(function(data) {
+        const needsForeign = showForeignFilter && $scope.selection.filterQuery.foreign !== 'false';
+        DataikuAPI.agentTools.listHeads($stateParams.projectKey, needsForeign).success(function(data) {
+            $scope.listItems = data
+            $scope.listItems.forEach((item) => {
+                item.foreign = item.projectKey !== $stateParams.projectKey;
+                item.name = item.foreign ? (item.projectKey + '.' + item.name) : item.name
+            });
             $scope.agentsUiData = data.reduce((acc, agent) => {
                 acc[agent.id] = AgentToolService.getAgentToolTypesUiData(agent);
                 return acc;
@@ -797,7 +944,7 @@ app.controller("AgentToolsListController", function($scope, $controller, $stateP
     }
 });
 
-app.controller('NewAgentToolController', function ($scope, $state, $stateParams, DataikuAPI, AgentToolService, WT1) {
+app.controller('NewAgentToolController', function ($scope, $state, $stateParams, DataikuAPI, AgentToolService, CreateModalFromTemplate, WT1) {
     $scope.selection = $.extend(
         {
             filterParams: {
@@ -819,14 +966,25 @@ app.controller('NewAgentToolController', function ($scope, $state, $stateParams,
         if (!$scope.newAgentTool.name) {
             $scope.newAgentTool.name = $scope.defaultName;
         }
+
+        // open another modal to choose between local and remote MCP client tools
+        if ($scope.selection.selectedObject.type === "VirtualTypeForMCPClients") {
+            CreateModalFromTemplate('/templates/agent-tools/new-mcp-tool-modal.html', $scope, "NewMCPToolController");
+            return;
+        }
+
         const quickTestQuery = AgentToolService.getAgentToolQuickTestQueryForType($scope.selection.selectedObject.type);
         DataikuAPI.agentTools
             .create($stateParams.projectKey, { ...$scope.newAgentTool, type: $scope.selection.selectedObject.type, quickTestQuery: quickTestQuery })
             .success(function (data) {
-                WT1.event(
-                    'agent-tool-create', {
-                        agentToolType: $scope.selection.selectedObject.type,
-                    });
+                const wt1AgentToolCreate = {
+                    agentToolType: $scope.selection.selectedObject.type,
+                };
+                const pluginId = AgentToolService.getCustomAgentPluginId($scope.selection.selectedObject.type);
+                if (pluginId) {
+                    wt1AgentToolCreate.pluginId = pluginId;
+                }
+                WT1.event('agent-tool-create', wt1AgentToolCreate);
                 $state.go('projects.project.agenttools.agenttool', { agentToolId: data.id });
                 $scope.dismiss();
             })
@@ -840,6 +998,91 @@ app.controller('NewAgentToolController', function ($scope, $state, $stateParams,
     });
 });
 
+app.controller('NewMCPToolController', function ($scope, $state, $stateParams, DataikuAPI, AgentToolService, WT1) {
+    $scope.toolTypes = [
+        {
+            "id": "RemoteMCPClient",
+            "icon": "dku-icon-mcp-client-remote-24",
+            "title": "Remote MCP server",
+            "desc": "Access tools from a remote MCP server through an existing connection.",
+        },
+        {
+            "id": "GenericStdioMCPClient",
+            "icon": "dku-icon-mcp-client-local-24",
+            "title": "Local MCP server",
+            "desc": "Configure and run a local MCP server in your environment.",
+        },
+    ]
+    $scope.uiState = {
+        selectedType: "RemoteMCPClient",
+        disabledlocalMCPReason: null,
+    };
+    $scope.params = {
+        // remote MCP client only
+        connectionName: null,
+
+        // local MCP client only
+        codeEnvSelection: {
+            "envMode": "INHERIT"
+        },
+    };
+
+    DataikuAPI.connections.getNames("RemoteMCP").success(function(data) {
+        $scope.availableRemoteMCPConnections = data;
+    }).error(setErrorInScope.bind($scope));
+
+    DataikuAPI.agentTools.canCreateLocalMCPTool().success(function(data) {
+        if (!data.enabled) {
+            $scope.uiState.disabledlocalMCPReason = data.reason;
+        }
+    }).error(setErrorInScope.bind($scope));
+
+    $scope.isToolTypeEnabled = function(toolTypeId) {
+        return toolTypeId !== "GenericStdioMCPClient" || !$scope.uiState.disabledlocalMCPReason;
+    };
+
+    $scope.selectToolTypeClick = function(toolTypeId) {
+        if (!$scope.isToolTypeEnabled(toolTypeId)) {
+            return;
+        }
+        $scope.uiState.selectedType = toolTypeId;
+    };
+
+    $scope.selectToolTypeKeydown = function(event, toolTypeId) {
+        if (!$scope.isToolTypeEnabled(toolTypeId)) {
+            return;
+        }
+        if (event.key === "Enter") {
+            $scope.uiState.selectedType = toolTypeId;
+        }
+    };
+
+    $scope.create = function () {
+        const newAgentToolType = $scope.uiState.selectedType;
+        const quickTestQuery = AgentToolService.getAgentToolQuickTestQueryForType(newAgentToolType);
+        let creationParams = {};
+        if (newAgentToolType === "GenericStdioMCPClient") {
+            creationParams.codeEnvSelection = $scope.params.codeEnvSelection;
+        } else {
+            creationParams.connectionName = $scope.params.connectionName;
+        }
+        DataikuAPI.agentTools
+            .create($stateParams.projectKey, {
+                ...$scope.newAgentTool,
+                type: newAgentToolType,
+                quickTestQuery: quickTestQuery,
+                creationParams: creationParams,
+            })
+            .success(function (data) {
+                WT1.event('agent-tool-create', {
+                    agentToolType: newAgentToolType,
+                });
+                $state.go('projects.project.agenttools.agenttool', { agentToolId: data.id });
+                $scope.dismiss();
+            }).error(setErrorInScope.bind($scope));
+    };
+});
+
 app.component("agentToolQuicktest", {
     bindings: {
         projectKey: "<",
@@ -850,7 +1093,7 @@ app.component("agentToolQuicktest", {
         datasetSchema: "<",
     },
     templateUrl : '/templates/agent-tools/quicktest.html',
-    controller: function($scope, $timeout, $q, DataikuAPI, AgentToolService, ClipboardUtils, CodeMirrorSettingService, SavedModelHelperService, WT1) {
+    controller: function($scope, $timeout, $q, DataikuAPI, AgentToolService, ClipboardUtils, CodeMirrorSettingService, SavedModelHelperService, TraceExplorerService, WT1) {
         const $ctrl = this;
         $ctrl.codeMirrorSettingService = CodeMirrorSettingService;
         $ctrl.quickTestResponse = null;
@@ -904,10 +1147,14 @@ app.component("agentToolQuicktest", {
                 $ctrl.quickTestRequestCancellable = $q.defer();
 
                 DataikuAPI.agentTools.test($ctrl.projectKey, $ctrl.agentToolId, $ctrl.quickTestQuery, $ctrl.quickTestRequestCancellable.promise).then((response) => {
-                    WT1.event(
-                        'agent-tool-test-run', {
-                            agentToolType: $ctrl.agentToolType,
-                        });
+                    const wt1AgentToolTestRun = {
+                        agentToolType: $ctrl.agentToolType,
+                    };
+                    const pluginId = AgentToolService.getCustomAgentPluginId($ctrl.agentToolType);
+                    if (pluginId) {
+                        wt1AgentToolTestRun.pluginId = pluginId;
+                    }
+                    WT1.event('agent-tool-test-run', wt1AgentToolTestRun);
                     $ctrl.quickTestResponse = response.data;
                     $ctrl.graphicalTrace = SavedModelHelperService.generateAsciiTreeFromTrace(response.data.fullTrace);
                     $scope.activeTab = 'response';
@@ -973,11 +1220,15 @@ app.component("agentToolQuicktest", {
                 });
         }
 
-        $ctrl.copyTraceToClipboard = function(value) {
+        $ctrl.copyTraceToClipboard = function() {
             $scope.traceCopied = true;
             ClipboardUtils.copyToClipboard(JSON.stringify($ctrl.quickTestResponse.fullTrace));
             $timeout(() => { $scope.traceCopied = false}, 5000);
         };
+        
+        $ctrl.openTraceExplorer = function() {
+            TraceExplorerService.openTraceExplorer($ctrl.quickTestResponse.fullTrace)
+        };
 
         $scope.$on('agentToolSavedEvent', () => {
             // only reload the descriptor if previously loaded manually
@@ -1002,7 +1253,7 @@ app.component("agentToolQuicktest", {
 });
 
 app.directive('agentToolRightColumnSummary', function($controller, $state, $stateParams, $rootScope, DataikuAPI, CreateModalFromTemplate, QuickView,
-    ActiveProjectKey, ActivityIndicator, TopNav, WT1, GlobalProjectActions) {
+    ActiveProjectKey, ActivityIndicator, TopNav, WT1, GlobalProjectActions, SmartId) {
 
     return {
         templateUrl :'/templates/agent-tools/right-column-summary.html',
@@ -1024,8 +1275,10 @@ app.directive('agentToolRightColumnSummary', function($controller, $state, $stat
             scope.refreshData = function() {
                 const projectKey = scope.selection.selectedObject.projectKey;
                 const name = scope.selection.selectedObject.name;
+                const toolId = scope.selection.selectedObject.id;
+                scope.canAccessObject = false;
 
-                DataikuAPI.agentTools.getFullInfo(ActiveProjectKey.get(), scope.selection.selectedObject.id).then(function({data}){
+                DataikuAPI.agentTools.getFullInfo(ActiveProjectKey.get(), SmartId.create(toolId, projectKey)).then(function({data}){
                     if (!scope.selection.selectedObject || scope.selection.selectedObject.projectKey != projectKey || scope.selection.selectedObject.name != name) {
                         return;
                     }
@@ -1034,6 +1287,9 @@ app.directive('agentToolRightColumnSummary', function($controller, $state, $stat
                     scope.agentTool = data.agentTool;
                     scope.selection.selectedObject.interest = data.interest;
                     scope.selection.selectedObject.agentToolType = scope.agentTool.type;
+                    scope.isLocalAgentTool = projectKey == ActiveProjectKey.get();
+                    scope.objectAuthorizations = data.objectAuthorizations;
+                    scope.canAccessObject = true;
                 }).catch(setErrorInScope.bind(scope));
             };
 
@@ -1282,80 +1538,69 @@ app.component("agentToolSelector", {
     bindings: {
         availableAgentTools: "<",
         selectedAgentTool: "=",
-        contextProject: "<",
-        onChange: "<",
-        defaultOpened: '<',
-        disableAlertWarning: '<',
-        placeholder: '<', // Optional, defaults to "Nothing selected"
     },
     template: `
 <basic-select ng-if="$ctrl.availableAgentTools"
           items="$ctrl.availableAgentTools | orderBy: 'name'"
-          ng-model="$ctrl.selectedAgentTool"
-          bind-value="id"
+          ng-model="$ctrl.selectedAgentTool.toolRef"
+          bind-value-fn="$ctrl.valueFn"
+          ng-change="$ctrl._updateComponentState()"
           bind-label="name"
           group-by-fn="$ctrl.niceAgentToolType"
           searchable="true"
-          placeholder="{{ $ctrl.placeholder }}"
+          placeholder="Nothing selected"
           invalidate-on-ghosts="true"
-          default-opened="$ctrl.defaultOpened"
           ghost-items-tooltip="Agent tool not available. It has been disabled, retired, or is not supported for this feature. Choose another agent tool or contact your administrator."
           custom-ng-select-class="ng-dropdown-panel--500 
                                   ng-select--custom-dropdown-height 
                                   ng-select--custom-x-overflow
                                   ng-select--custom-reset-default-padding"></basic-select>
-<div class="mbot0 mtop8 alert alert-warning" ng-if="$ctrl.selectedToolIsForeign()">
-Unable to display this tool properly: it was defined in the project {{$ctrl.getToolProjectKey()}} and not shared with the current project. Note that the tool or the project may not exist (e.g. if the current project was imported but the other project wasn't), or may have been deleted.
-</div>
-<div class="mbot0 mtop8 alert alert-warning" ng-if="!$ctrl.selectedToolIsForeign() && !$ctrl.isAvailable && !$ctrl.disableAlertWarning">
-Agent tool not available. It has been disabled, retired, or is not supported for this feature. Choose another Agent tool or contact your administrator. <strong>{{$ctrl.selectedAgentTool.id}}</strong>
+<div ng-if="!$ctrl.isAvailable && $ctrl.selectedAgentTool.toolRef">
+    <div class="mbot0 mtop8 alert alert-warning" ng-if="$ctrl.isForeign">
+    Unable to display this tool properly: it was defined in the project {{$ctrl.foreignProjectKey}} and not shared with the current project. Note that the tool or the project may not exist (e.g. if the current project was imported but the other project wasn't), or may have been deleted.
+    </div>
+    <div class="mbot0 mtop8 alert alert-warning" ng-if="!$ctrl.isForeign">
+    Agent tool not available. It has been disabled, retired, or is not supported for this feature. Choose another Agent tool or contact your administrator. <strong>{{$ctrl.selectedAgentTool.id}}</strong>
+    </div>
 </div>`,
-    controller: function ($scope, $filter) {
+    controller: function ($filter, SmartId, $stateParams) {
+        const niceAgentToolTypeFilter = $filter('niceAgentToolType');
         const $ctrl = this;
 
-        $scope.$watch(
-            function () {
-                return $ctrl.selectedAgentTool;
-            },
-            function (newVal, oldVal) {
-                $ctrl.updateAvailability();
-                if (newVal !== oldVal && $ctrl.onChange) {
-                    $ctrl.onChange();
+        $ctrl.$onChanges = (changes) => {
+            if (changes.availableAgentTools) {
+                $ctrl._updateComponentState();
+            }
+        };
+
+        $ctrl.valueFn = function (value) {
+            return SmartId.fromTor(value);
+        };
+
+        $ctrl._updateComponentState = () => {
+            $ctrl.isAvailable = true;
+            $ctrl.isForeign = false;
+            $ctrl.foreignProjectKey = null;
+
+            if (!$ctrl.selectedAgentTool || !$ctrl.selectedAgentTool.toolRef) {
+                return;
+            }
+
+            if ($ctrl.availableAgentTools) {
+                $ctrl.isAvailable = $ctrl.availableAgentTools.some((l) => SmartId.fromTor(l) === $ctrl.selectedAgentTool.toolRef);
+            }
+
+            if (!$ctrl.isAvailable) {
+                const toolProjectKey = SmartId.resolve($ctrl.selectedAgentTool.toolRef).projectKey;
+                $ctrl.isForeign = toolProjectKey !== $stateParams.projectKey;
+                if ($ctrl.isForeign) {
+                    $ctrl.foreignProjectKey = toolProjectKey;
                 }
             }
-        );
-
-        $ctrl.$onChanges = () => {
-            if (typeof $ctrl.placeholder === 'undefined' || $ctrl.placeholder === null) {
-                $ctrl.placeholder = "Nothing selected";
-            }
-
-            $ctrl.updateAvailability();
-        };
-
-        $ctrl.getToolProjectKey = function () {
-            if ($ctrl.selectedAgentTool && $ctrl.selectedAgentTool.includes(".")) {
-                return $ctrl.selectedAgentTool.split(".")[0];
-            }
-        };
-
-        $ctrl.selectedToolIsForeign = function () {
-            const toolProjectKey = $ctrl.getToolProjectKey();
-            if (toolProjectKey && toolProjectKey !== $ctrl.contextProject) {
-                return true;
-            }
-            return false;
-
-        };
-        $ctrl.isAvailable = true;
-        $ctrl.updateAvailability = () => {
-            if ($ctrl.availableAgentTools && $ctrl.selectedAgentTool) {
-                $ctrl.isAvailable = !!$ctrl.availableAgentTools.find((l) => l.id === $ctrl.selectedAgentTool);
-            }
         };
 
         $ctrl.niceAgentToolType = (agentToolOption) => {
-            return $filter('niceAgentToolType')(agentToolOption.type);
+            return niceAgentToolTypeFilter(agentToolOption.type);
         };
 
     },
