@@ -777,8 +777,14 @@ app.controller('WikiTaxonomyController', function($scope, $stateParams, $state,
         }
     });
     
-    window.addEventListener("beforeunload", function() {
+    const beforeUnloadEventHandler = function() {
         $scope.wikiScope.saveFoldingState(); // When leaving the wiki by closing the tab/window, or manually changing the URL
+    }
+
+    window.addEventListener("beforeunload", beforeUnloadEventHandler);
+
+    $scope.$on('$destroy', function() {
+        window.removeEventListener("beforeunload", beforeUnloadEventHandler);
     });
 
     $scope.wikiScope.saveFoldingState = function() {
