@@ -13,7 +13,7 @@
      * @typedef {import("../../../../../../../../server/src/frontend/src/app/features/simple-report/services/chart-features/chart-features.type").ChartFeatureReturnType} ChartFeatureReturnType
      * @returns {ChartFeatureReturnType}
      */
-    function chartFeatures(WebAppsService, ChartStoreFactory, ChartDimension, ChartsStaticData, ChartUADimension, ChartTypeChangeUtils, D3ChartZoomControl, CHART_TYPES, CHART_VARIANTS, CHART_AXIS_TYPES, NumberFormatter, AxisTicksConfigMode, VALUES_DISPLAY_MODES, translate) {
+    function chartFeatures(WebAppsService, ChartStoreFactory, ChartDimension, ChartsStaticData, ChartUADimension, ChartTypeChangeUtils, D3ChartZoomControl, CHART_TYPES, CHART_VARIANTS, CHART_AXIS_TYPES, NumberFormatter, AxisTicksConfigMode, VALUES_DISPLAY_MODES, translate, $rootScope) {
 
         function isChartTypeInList(chartType, chartTypeList) {
             return chartTypeList.includes(chartType);
@@ -38,10 +38,10 @@
                 if (chartDef.facetDimension.length) {
                     return false;
                 }
-                return isChartTypeInList(chartDef.type, [
-                    CHART_TYPES.STACKED_COLUMNS, CHART_TYPES.STACKED_AREA, CHART_TYPES.GROUPED_COLUMNS,
+                return (isChartTypeInList(chartDef.type, [
+                    CHART_TYPES.STACKED_COLUMNS, CHART_TYPES.STACKED_AREA,
                     CHART_TYPES.LINES, CHART_TYPES.PIVOT_TABLE, CHART_TYPES.GROUPED_XY, CHART_TYPES.KPI
-                ]);
+                ]) || (chartDef.type === CHART_TYPES.GROUPED_COLUMNS && chartDef.variant !== CHART_VARIANTS.waterfall));
             },
 
             canExportToImage: chartDef => {
@@ -70,20 +70,33 @@
                 }
             },
 
-            canAnimate: chartType => isChartTypeInList(chartType, [
+            canDrillHierarchy: chartType => isChartTypeInList(chartType, [
+                CHART_TYPES.GROUPED_COLUMNS, CHART_TYPES.STACKED_COLUMNS, CHART_TYPES.STACKED_BARS, CHART_TYPES.RADAR,
+                CHART_TYPES.LINES, CHART_TYPES.MULTI_COLUMNS_LINES, CHART_TYPES.PIE, CHART_TYPES.STACKED_AREA, CHART_TYPES.BOXPLOTS,
+                CHART_TYPES.BINNED_XY, CHART_TYPES.GROUPED_XY, CHART_TYPES.LIFT, CHART_TYPES.PIVOT_TABLE, CHART_TYPES.TREEMAP
+            ]),
+
+            canDisplayBreadcrumb: chartType => svc.canDrillHierarchy(chartType) && chartType !== CHART_TYPES.PIVOT_TABLE,
+
+            shouldDisplayBreadcrumb: chartDef => svc.canDisplayBreadcrumb(chartDef.type) && chartDef.drilldownOptions.displayBreadcrumb,
+
+            canAnimate: (chartType, chartVariant) => isChartTypeInList(chartType, [
                 CHART_TYPES.MULTI_COLUMNS_LINES, CHART_TYPES.GROUPED_COLUMNS, CHART_TYPES.STACKED_COLUMNS,
                 CHART_TYPES.STACKED_BARS, CHART_TYPES.LINES, CHART_TYPES.STACKED_AREA, CHART_TYPES.PIE,
                 CHART_TYPES.BINNED_XY, CHART_TYPES.GROUPED_XY, CHART_TYPES.LIFT
-            ]),
+            ]) && chartVariant !== CHART_VARIANTS.waterfall,
 
-            canFacet: (chartType, webAppType) => {
+            canFacet: (chartType, webAppType, chartVariant) => {
+                if (chartType === CHART_TYPES.GROUPED_COLUMNS) {
+                    return chartVariant !== CHART_VARIANTS.waterfall;
+                }
                 if (chartType === CHART_TYPES.WEBAPP) {
                     const loadedDesc = WebAppsService.getWebAppLoadedDesc(webAppType) || {};
                     const pluginChartDesc = (loadedDesc.desc && loadedDesc.desc.chart) || {};
                     return pluginChartDesc.canFacet === true;
                 } else {
                     return isChartTypeInList(chartType, [
-                        CHART_TYPES.MULTI_COLUMNS_LINES, CHART_TYPES.GROUPED_COLUMNS, CHART_TYPES.STACKED_COLUMNS,
+                        CHART_TYPES.MULTI_COLUMNS_LINES, CHART_TYPES.STACKED_COLUMNS,
                         CHART_TYPES.STACKED_BARS, CHART_TYPES.LINES, CHART_TYPES.STACKED_AREA, CHART_TYPES.PIE,
                         CHART_TYPES.BINNED_XY, CHART_TYPES.GROUPED_XY, CHART_TYPES.LIFT
                     ]);
@@ -107,7 +120,7 @@
                 return !_.isNil(chartDef) && !_.isNil(zoomControlInstanceId) && (svc.canHaveEChartsZoomControls(chartDef) || svc.hasScatterZoomControlEnabled(chartDef.type, zoomControlInstanceId) || svc.canHaveMapsZoomControls(chartDef));
             },
 
-            canHaveEChartsZoomControls: chartDef => chartDef.type === CHART_TYPES.TREEMAP && chartDef.yDimension.length > 0 && chartDef.genericMeasures.length > 0,
+            canHaveEChartsZoomControls: chartDef => chartDef.type === CHART_TYPES.TREEMAP && !!ChartDimension.getYDimensions(chartDef).length && !!chartDef.genericMeasures.length,
 
             canHaveMapsZoomControls: chartDef => svc.isMap(chartDef.type) && (chartDef.geometry.length > 0 || chartDef.geoLayers.length > 1),
 
@@ -183,14 +196,14 @@
             },
 
             canSelectPivotRowOrColumnDisplayMode: chartDef => chartDef != null && chartDef.type === CHART_TYPES.PIVOT_TABLE &&
-                chartDef.yDimension && chartDef.yDimension.length > 0 && chartDef.xDimension && chartDef.xDimension.length > 0,
+                !!ChartDimension.getYDimensions(chartDef).length && !!ChartDimension.getXDimensions(chartDef).length,
 
             canCustomizePivotFormatting: chartDef => chartDef != null && chartDef.type === CHART_TYPES.PIVOT_TABLE &&
-                ((chartDef.yDimension && chartDef.yDimension.length > 0) || (chartDef.xDimension && chartDef.xDimension.length > 0))
+                (!!ChartDimension.getYDimensions(chartDef).length || !!ChartDimension.getXDimensions(chartDef).length)
                 && (chartDef.genericMeasures && chartDef.genericMeasures.length),
 
             canCustomizePivotRowHeaders: chartDef => svc.canCustomizePivotFormatting(chartDef) &&
-                (chartDef.yDimension && chartDef.yDimension.length > 0) || (chartDef.genericMeasures && chartDef.genericMeasures.length > 1),
+                !!ChartDimension.getYDimensions(chartDef).length || (chartDef.genericMeasures && chartDef.genericMeasures.length > 1),
 
             isColoredPivotTable: chartDef => chartDef != null && chartDef.type === CHART_TYPES.PIVOT_TABLE && (
                 chartDef.colorMeasure && chartDef.colorMeasure.length ||
@@ -228,6 +241,10 @@
                     [CHART_TYPES.PIE, CHART_TYPES.PIVOT_TABLE, CHART_TYPES.RADAR, CHART_TYPES.SANKEY, CHART_TYPES.TREEMAP, CHART_TYPES.SCATTER, CHART_TYPES.SCATTER_MULTIPLE_PAIRS, CHART_TYPES.BOXPLOTS, CHART_TYPES.GROUPED_XY]) || chartDef.hexbin)
                 && chartDef.variant !== CHART_VARIANTS.binnedXYHexagon,
 
+            chartOnlySupportsOneTickPerBin: (chartDef) => {
+                return chartDef?.type === CHART_TYPES.GROUPED_COLUMNS && chartDef?.variant === CHART_VARIANTS.waterfall;
+            },
+
             canModifyAxisRange: (chartDef, axis) => {
                 if (!canSetCustomAxisRange(chartDef)) {
                     return false;
@@ -258,7 +275,7 @@
             ]),
 
             isPivotTableWithNoDimension: (chartDef) => {
-                return chartDef.type === CHART_TYPES.PIVOT_TABLE && chartDef.xDimension.length == 0 && chartDef.yDimension.length == 0;
+                return chartDef.type === CHART_TYPES.PIVOT_TABLE && !ChartDimension.getXDimensions(chartDef).length && !ChartDimension.getYDimensions(chartDef).length;
             },
 
             canHaveConditionalFormatting: chartType => isChartTypeInList(chartType, [
@@ -326,7 +343,7 @@
                             return axisPair && ChartUADimension.isTrueNumerical(axisPair.uaYDimension[0]);
                         }
                         return chartDef.type !== CHART_TYPES.STACKED_BARS
-                            && chartDef.variant !== CHART_VARIANTS.stacked100;
+                            && chartDef.variant !== CHART_VARIANTS.stacked100 && chartDef.variant !== CHART_VARIANTS.waterfall;
                     default:
                         return false;
                 }
@@ -338,7 +355,8 @@
 
             canSelectGridlinesAxis: chartType => svc.canSetLeftRightYAxes(chartType) || chartType === CHART_TYPES.SCATTER_MULTIPLE_PAIRS,
 
-            canHaveVerticalGridlines: chartDef => !(chartDef.facetDimension.length && chartDef.singleXAxis) && !(chartDef.type === CHART_TYPES.BOXPLOTS && !chartDef.boxplotBreakdownDim.length),
+            canHaveVerticalGridlines: chartDef => !(chartDef.facetDimension.length && chartDef.singleXAxis)
+                && !(chartDef.type === CHART_TYPES.BOXPLOTS && !chartDef.boxplotBreakdownDim.length && !chartDef.boxplotBreakdownHierarchyDimension.length),
 
             getVLinesDisabledTooltip: chartType => chartType === CHART_TYPES.BOXPLOTS
                 ? translate('CHARTS.SETTINGS.FORMAT.GRIDLINES.NO_X_DIMENSION', 'Add an X dimension to be able to display vertical gridlines')
@@ -348,6 +366,17 @@
 
             isKPIChart: chartType => chartType === CHART_TYPES.KPI,
 
+            canUseUnaggregatedMeasures: (chartDef, column, measureType, measureDisplayType, contextualMenuMeasureType) => {
+                return $rootScope?.featureFlagEnabled?.('unaggregatedMeasures') && !!column // can't use unaggregated mode for count of records
+                    && ([
+                        // CHART_TYPES.MULTI_COLUMNS_LINES, CHART_TYPES.LINES,
+                        CHART_TYPES.GROUPED_COLUMNS].includes(chartDef.type)
+                        || (chartDef.type === CHART_TYPES.MULTI_COLUMNS_LINES && measureDisplayType === 'column')
+                        || (chartDef.type === CHART_TYPES.STACKED_COLUMNS && chartDef.variant !== CHART_VARIANTS.stacked100)
+                    )
+                    && measureType === 'NUMERICAL' && (!contextualMenuMeasureType || contextualMenuMeasureType === 'generic' || contextualMenuMeasureType === 'valuesInChart' || contextualMenuMeasureType === 'tooltip');
+            }, // for now unaggregated mode available only in chart def measures
+
             isGaugeChart: chartType => chartType === CHART_TYPES.GAUGE,
 
             isKPILikeChart: chartType => isChartTypeInList(chartType, [
@@ -380,7 +409,7 @@
 
                 addMeasureComputationMode('NORMAL');
 
-                const nbGDims = chartDef.genericDimension0.length + chartDef.genericDimension1.length;
+                const nbGDims = chartDef.genericDimension0.length + chartDef.genericHierarchyDimension.length + chartDef.genericDimension1.length;
 
                 switch (chartDef.type) {
                     case CHART_TYPES.GROUPED_COLUMNS:
@@ -403,7 +432,7 @@
                         addMeasureComputationMode('CUMULATIVE_PERCENTAGE');
                         break;
                     case CHART_TYPES.PIVOT_TABLE:
-                        if (chartDef.yDimension.length <= 1 && chartDef.xDimension.length <= 1) {
+                        if (ChartDimension.getXDimensions(chartDef).length <= 1 && ChartDimension.getYDimensions(chartDef).length <= 1) {
                             addMeasureComputationMode('PERCENTAGE');
                             addMeasureComputationMode('AVG_RATIO');
                             addMeasureComputationMode('CUMULATIVE');
@@ -415,7 +444,7 @@
                         addMeasureComputationMode('PERCENTAGE');
                         break;
                     case CHART_TYPES.TREEMAP:
-                        if (chartDef.yDimension.length <= 2) {
+                        if (ChartDimension.getYDimensions(chartDef).length <= 2) {
                             addMeasureComputationMode('PERCENTAGE');
                         }
                         break;
@@ -425,7 +454,10 @@
             },
 
             canSetAlongDimension: function(chartDef, computeMode) {
-                if (((chartDef.genericDimension0.length && chartDef.genericDimension1.length) || (chartDef.xDimension.length && chartDef.yDimension.length) || (chartDef.type == CHART_TYPES.TREEMAP && chartDef.yDimension.length == 2))
+                const genericDimension0 = ChartDimension.getGenericDimension(chartDef);
+                const xDimension = ChartDimension.getXDimension(chartDef);
+                const yDimensions = ChartDimension.getYDimensions(chartDef);
+                if (((genericDimension0 && chartDef.genericDimension1.length) || (xDimension && yDimensions.length) || (chartDef.type == CHART_TYPES.TREEMAP && yDimensions.length == 2))
                     && (computeMode == 'PERCENTAGE' || computeMode == 'AVG_RATIO' || computeMode == 'CUMULATIVE_PERCENTAGE')) {
                     return true;
                 }
@@ -433,18 +465,28 @@
             },
 
             getDimensionsForComputeAlong: chartDef => {
-                switch (chartDef.type) {
-                    case CHART_TYPES.PIVOT_TABLE:
-                        // PIVOT order of insertion of the axes is defined as follows: [...xDimension, ...yDimension]
-                        return [[0, chartDef.xDimension[0].column], [1, chartDef.yDimension[0].column]];
-                    case CHART_TYPES.SANKEY:
-                    case CHART_TYPES.TREEMAP:
-                        // TREEMAP order of insertion of the axes is defined as follows: [...yDimension[0], ...yDimension[1]]
-                        return [[0, chartDef.yDimension[0].column], [1, chartDef.yDimension[1].column]];
-                    default:
-                        // default order of insertion of the axes is defined as follows: [...genericDimension0, ...genericDimension1]
-                        return [[0, chartDef.genericDimension0[0].column], [1, chartDef.genericDimension1[0].column]];
+                if (chartDef.type === CHART_TYPES.PIVOT_TABLE) {
+                    const xDimension = ChartDimension.getXDimension(chartDef);
+                    const yDimension = ChartDimension.getYDimension(chartDef);
+                    if (!xDimension || !yDimension) {
+                        return [];
+                    }
+                    // PIVOT order of insertion of the axes is defined as follows: [...xDimension, ...yDimension]
+                    return [[0, xDimension.column], [1, yDimension.column]];
                 }
+                if (chartDef.type === CHART_TYPES.SANKEY || chartDef.type === CHART_TYPES.TREEMAP) {
+                    const yDimensions = ChartDimension.getYDimensions(chartDef);
+                    if (yDimensions.length < 2) {
+                        return [];
+                    }
+                    return [[0, yDimensions[0].column], [1, yDimensions[1].column]];
+                }
+                const genericDimension = ChartDimension.getGenericDimension(chartDef);
+                if (!genericDimension || !chartDef.genericDimension1 || !chartDef.genericDimension1[0]) {
+                    return [];
+                }
+                // default order of insertion of the axes is defined as follows: [...genericDimension0, ...genericDimension1]
+                return [[0, genericDimension.column], [1, chartDef.genericDimension1[0].column]];
             },
 
             isEChart: (chartDef) => {
@@ -645,8 +687,8 @@
                 return chartType !== CHART_TYPES.SCATTER && chartType !== CHART_TYPES.SCATTER_MULTIPLE_PAIRS;
             },
 
-            canFilterInTooltip: (chartType) => {
-                return ![CHART_TYPES.SCATTER, CHART_TYPES.SCATTER_MULTIPLE_PAIRS].includes(chartType);
+            canFilterInTooltip: (chartDef) => {
+                return ![CHART_TYPES.SCATTER, CHART_TYPES.SCATTER_MULTIPLE_PAIRS].includes(chartDef.type);
             },
 
             canIgnoreEmptyBinsForColorDimension: (chartDef) => {
@@ -677,7 +719,11 @@
 
                     return {
                         hasLessRecordsThanSampling,
-                        message: `${NumberFormatter.longSmartNumberFilter()(numberOfRecords)} points limit reached. You can modify the limit in the "Misc" section of your chart.`
+                        message: translate(
+                            'CHARTS.HEADER.BADGE.TRUNCATED.TOOLTIP',
+                            `${NumberFormatter.longSmartNumberFilter()(numberOfRecords)} points limit reached. You can modify the limit in the "Misc" section of your chart.`,
+                            { maxNbPoints: NumberFormatter.longSmartNumberFilter()(numberOfRecords) }
+                        )
                     };
                 } else {
                     return null;
