@@ -8,6 +8,7 @@
             sessionId: '<',
             availableLlms: '<',
             hasActiveLlm: '<',
+            warnAboutRaLlmSearchMode: '<',
             activeLlmIcon: '<',
             emptyStateMessage: '<',
             onBeforeSendMessage: '&?',
@@ -18,7 +19,7 @@
             processResponse: '&?'
         },
         templateUrl: '/static/dataiku/shared/components/prompt-chat/prompt-chat.component.html',
-        controller: function($rootScope, $scope, $timeout, PromptUtils, PromptChatService, ClipboardUtils, Logger, $dkuSanitize) {
+        controller: function($rootScope, $scope, $timeout, PromptUtils, PromptChatService, TraceExplorerService, ClipboardUtils, Logger, $dkuSanitize) {
             const $ctrl = this;
             const entryElId = 'prompt-chat-entry';
 
@@ -58,6 +59,8 @@
                     ...($ctrl.chatSessions[activeSessionId] || {}),
                     streamedMessage: '',
                     rawMessage: '',
+                    rawReasoning: '',
+                    reasoning: '',
                     currentlyStreaming: true,
                     useMarkdown: true,
                     streamingSupported: true,
@@ -71,6 +74,7 @@
                 const CHAR_LIMIT_FOR_MARKDOWN = 500;
                 const THROTTLE_MIN_LENGTH = 10000;
                 let previousLength = 0;
+                let currentReasoningIndex;
                 const callback = chunk => {
                     scrollChat(activeSessionId, false);
 
@@ -78,6 +82,42 @@
                     const data = chunk['data'];
                     switch(event) {
                         case 'completion-chunk': {
+                            if (data.artifacts && data.artifacts.length) {
+                                data.artifacts.forEach((artifact) => {
+                                    if (artifact.type === 'REASONING' && artifact.parts  && artifact.parts.length) {
+                                        artifact.parts.forEach((artifactPart) => {
+                                            if (artifactPart.type === 'TEXT') {
+                                                if (currentReasoningIndex == null) {
+                                                    currentReasoningIndex = artifactPart.index;
+                                                }
+                                                if (currentReasoningIndex !== artifactPart.index) {
+                                                    currentReasoningIndex = artifactPart.index;
+                                                    activeSession.rawReasoning += "\n\n";
+                                                }
+                                                activeSession.rawReasoning += artifactPart.text;
+
+                                                // throttle text streaming if the string is too long
+                                                if (activeSession.useMarkdown) {
+                                                    // if there is an error parsing the HTML (e.g., LLM is spitting out garbage text), stop using markdown
+                                                    try {
+                                                        activeSession.reasoning = $dkuSanitize(marked(activeSession.rawReasoning, {
+                                                            breaks: true
+                                                        }));
+                                                    } catch(e) {
+                                                        Logger.error('Error parsing markdown HTML, switching to plain text', e);
+                                                        activeSession.useMarkdown = false;
+                                                    }
+                                                }
+                                                if (!activeSession.useMarkdown) {
+                                                    activeSession.reasoning = activeSession.rawReasoning;
+                                                }
+                                                $scope.$apply();
+                                                scrollChat(activeSessionId, false);
+                                            }
+                                        })
+                                    }
+                                })
+                            }
                             if (data.text) {
                                 const text = data.text;
                                 activeSession.rawMessage += text;
@@ -90,7 +130,6 @@
                                         activeSession.streamedMessage = $dkuSanitize(marked(activeSession.rawMessage, {
                                             breaks: true
                                         }));
-                                        $scope.$apply();
                                         previousLength = currentLength;
                                     } catch(e) {
                                         Logger.error('Error parsing markdown HTML, switching to plain text', e);
@@ -100,9 +139,8 @@
 
                                 if (!activeSession.useMarkdown) {
                                     activeSession.streamedMessage = activeSession.rawMessage;
-                                    $scope.$apply();
                                 }
-
+                                $scope.$apply();
                                 scrollChat(activeSessionId, false);
                             }
                             break;
@@ -263,11 +301,11 @@
 
             $ctrl.switchBranch = (message, position) => {
                 const parent = $ctrl.messages[message.parentId];
-                let nextMesssage = $ctrl.messages[parent.childrenIds[position]];
-                while (nextMesssage.childrenIds && nextMesssage.childrenIds.length) {
-                    nextMesssage = $ctrl.messages[nextMesssage.childrenIds[0]];
+                let nextMessage = $ctrl.messages[parent.childrenIds[position]];
+                while (nextMessage.childrenIds && nextMessage.childrenIds.length) {
+                    nextMessage = $ctrl.messages[nextMessage.childrenIds[0]];
                 }
-                $ctrl.lastMessageId = nextMesssage.id;
+                $ctrl.lastMessageId = nextMessage.id;
                 $ctrl.messageBranch = PromptChatService.buildCurrentChatBranch($ctrl.messages, $ctrl.lastMessageId);
             };
 
@@ -313,6 +351,10 @@
             $ctrl.copyMessageToClipboard = function(message) {
                 ClipboardUtils.copyToClipboard(message.error ? message.llmError : message.message.content, "Message copied to clipboard");
             };
+            
+            $ctrl.openTraceExplorer = function (message) {
+                TraceExplorerService.openTraceExplorer(message.fullTrace);
+            };
 
             $ctrl.stopStreaming = function() {
                 const sessionId = $ctrl.sessionId;
@@ -336,11 +378,12 @@
             };
 
             // Called if prompt chat kernel is manually stopped outside of prompt chat component
-            $rootScope.$on('llmStopDevKernel', (_, sessionId) => {
+            const llmStopDevKernelUnsubscribe = $rootScope.$on('llmStopDevKernel', (_, sessionId) => {
                 if ($ctrl.chatSessions[sessionId]?.currentlyStreaming) {
                     $ctrl.stopStreaming();
                 }
             });
+            $scope.$on('$destroy', llmStopDevKernelUnsubscribe);
 
             $ctrl.$onDestroy = () => {
                 PromptChatService.clearLogs();
