@@ -13,22 +13,22 @@ from dataiku.base.async_link import AsyncJavaLink
 from dataiku.base.socket_block_link import parse_javalink_args
 from dataiku.base.utils import watch_stdin
 from dataiku.core.knowledge_bank import MultipartContext, KnowledgeBank
-from dataiku.core.vector_stores.dku_vector_store import VectorStoreFactory
+from dataiku.core.vector_stores.dku_vector_store import VectorStoreFactory, DkuVectorStore
 from dataiku.huggingface.types import CompletionSettings, ChatMessage
+from dataiku.langchain.base_rag_handler import RetrievalSource
 from dataiku.langchain.dku_embeddings import DKUEmbeddings
-from dataiku.langchain.document_handler import DocumentHandler, RetrievalSource
+from dataiku.langchain.document_handler import DocumentHandler
 from dataiku.langchain.sources_handler import SourcesHandler
 from dataiku.langchain.content_part_types import TextPart, ImageRefPart, ImageRetrieval
 
-from dataiku.llm.evaluation.llm_metrics_input import LLMMetricInput
+from dataiku.llm.evaluation.genai_metrics_input import GenAIMetricInput
 from dataiku.llm.rag.guardrails import AnswerRelevancy, Faithfulness, Guardrail, MultimodalAnswerRelevancy, MultimodalFaithfulness
 from dataiku.llm.tracing import SpanBuilder, new_trace
-from dataiku.llm.types import LLMErrorType, ProcessSinglePromptCommand, RAGLLMSettings, RetrievableKnowledge, \
-    SimpleCompletionResponseOrError, Source, SourceItem, FilterDesc, SearchInputStrategy
+from dataiku.llm.types import LLMErrorType, ProcessSinglePromptCommand, RAGLLMSettings, SimpleCompletionResponseOrError, Source, FilterDesc, \
+    SearchInputStrategy, TrustedObject
 from dataikuapi.dss.llm import DSSLLM, DSSLLMCompletionQuery, DSSLLMStreamedCompletionChunk, DSSLLMStreamedCompletionFooter
 
 if TYPE_CHECKING:
-    from langchain_core.vectorstores import VectorStoreRetriever
     from langchain_core.documents import Document
 
     from dataiku.llm.evaluation.utils.ragas.ragas_utils_0_1_10 import RagasMetricsComputer
@@ -48,15 +48,14 @@ class RAGServer:
     retrieval_source: RetrievalSource
     document_handler: DocumentHandler
     sources_handler: SourcesHandler
-    embeddings_model_id: str
-    guardrails: List[Guardrail] = []
-    metrics_to_compute: List[str] = []
-    retriever: "VectorStoreRetriever"
+    guardrails: List[Guardrail]
+    metrics_to_compute: List[str]
     include_content_in_sources: bool
     print_sources: bool
     output_format: str
     filter: FilterDesc
-    run_counter: int = 1
+    dku_vector_store: DkuVectorStore
+    run_counter: int
 
     def __init__(self) -> None:
         self.started = False
@@ -72,28 +71,37 @@ class RAGServer:
         # single thread (including LLM calls)
         self.lock = threading.Lock()
 
+        # Initializing defaults
+        self.guardrails = []
+        self.metrics_to_compute = []
+        self.run_counter = 1
+
     def start(self, start_command: Dict) -> None:
         assert not self.started, "Already started"
+        self.rag_settings = start_command["ragSettings"]
 
         kb_full_id = start_command["knowledgeBankFullId"]
-        knowledge_bank = KnowledgeBank(kb_full_id)
+        knowledge_bank = KnowledgeBank(kb_full_id, context_project_key=dataiku.default_project_key())
         # the isolated folder will be cleaned up when the kernel dies
-        isolated_folder = knowledge_bank.load_into_isolated_folder()
+        llm_ref = start_command["llmRef"]
+        trusted_object: TrustedObject = {"smartRef": llm_ref["savedModelSmartId"], "type": "SAVED_MODEL"}
+        isolated_folder = knowledge_bank.load_into_isolated_folder(trusted_object=trusted_object)
         kb = isolated_folder.get_kb_desc()
 
         self.rag_settings = start_command["ragSettings"]
+        self.completion_settings = start_command["defaultCompletionSettings"]
         self.search_input_strategy = self.rag_settings["searchInputStrategySettings"]["strategy"]
         self.no_retrieval_key = start_command["noRetrievalKey"]
 
         self.embeddings = DKUEmbeddings(llm_id=kb["embeddingLLMId"])
         self.retrieval_source = RetrievalSource(self.rag_settings.get("retrievalSource", RetrievalSource.EMBEDDING))
-        retrieval_column = self.rag_settings.get("retrievalColumn") if self.retrieval_source == RetrievalSource.CUSTOM else None
-        self.document_handler = DocumentHandler(kb, self.rag_settings, self.retrieval_source, retrieval_column)
+        retrieval_columns = self.rag_settings.get("retrievalColumns") if self.retrieval_source != RetrievalSource.MULTIMODAL else None
+        self.document_handler = DocumentHandler(kb, self.rag_settings, self.retrieval_source, retrieval_columns)
         self.sources_handler = SourcesHandler(
             self.rag_settings.get("sourcesSettings", {}),
             kb.get("managedFolderId"),
             self.retrieval_source,
-            retrieval_column
+            retrieval_columns
         )
         # Guardrails settings initialization
         if self.rag_settings["ragSpecificGuardrails"]["faithfulnessSettings"]["enabled"]:
@@ -361,6 +369,9 @@ class RAGServer:
                 additional_search_kwargs: Dict = {}
                 if self.rag_settings.get("enforceDocumentLevelSecurity"):
                     caller_security_tokens = self._extract_caller_security_tokens(command)
+                    if len(caller_security_tokens) == 0:
+                        logger.warning("No caller security tokens provided: returning 0 documents")
+                        return []
                     self.dku_vector_store.add_security_filter(additional_search_kwargs, caller_security_tokens)
                 if self.filter is not None:
                     self.dku_vector_store.add_filter(additional_search_kwargs, self.filter)
@@ -393,8 +404,7 @@ class RAGServer:
         completion = self.llm.new_completion()
         _set_chat_messages(completion, final_messages)
 
-        # set a default value for max tokens
-        completion.settings["maxOutputTokens"] = 1024
+        completion.settings.update(self.completion_settings)
         # re-use the completion query settings (from the command)
         settings = process_prompt_command["settings"]
         completion.settings.update(settings)
@@ -513,10 +523,10 @@ class RAGServer:
                           sources: List[str],
                           trace: SpanBuilder) -> "SimpleCompletionResponseOrError":
         ragas_metrics_computer = self._get_ragas_metrics_computer(completion_settings)
-        mapped_input: LLMMetricInput = LLMMetricInput.from_single_entry(question, answer, None, sources)
+        mapped_input: GenAIMetricInput = GenAIMetricInput.from_single_entry(question, answer, None, sources, None, None)
 
         with trace.subspan("RAG_GUARDRAILS_METRICS_COMPUTE") as metrics_span:
-            overall_metrics, _ = ragas_metrics_computer.compute(interpreted_columns=mapped_input, metrics=self.metrics_to_compute, trace=metrics_span)
+            overall_metrics, _ = ragas_metrics_computer.compute_llm_metrics(metric_inputs=mapped_input, metrics=self.metrics_to_compute, trace=metrics_span)
 
         for guardrail in self.guardrails:
             metric_value = overall_metrics[guardrail.metric_key_name]
