@@ -1,4 +1,4 @@
-from .select_query import SelectQuery, Table, JoinTypes
+from .select_query import SelectQuery, Table, JoinTypes, CombinedQueries
 from .expression import Expression, Column, Constant, InlineSQL, Interval, Window, List, \
     DatePart, TimeUnit, ColumnType
 from .translate import Dialects, toSQL
\ No newline at end of file
